import near
import json
import base58

@near.export
def hello_world():
  input_bytes = near.input(0)
  base58_input = base58.b58encode(input_bytes).decode('ascii')
  near.log_utf8(f"hello_world(): input(0): {input_bytes.decode('ascii')}; b58(input(0)): {base58_input}")
  near.value_return(json.dumps({"input_length": len(input_bytes), "base58_input": base58_input}))

@near.export
def echo():
  input_bytes = near.input(0)
  base58_input = base58.b58encode(input_bytes).decode('ascii')
  near.log_utf8(f"hello_world(): input(0): {input_bytes.decode('ascii')}; b58(input(0)): {base58_input}")
  near.value_return(json.dumps({"input_length": len(input_bytes), "base58_input": base58_input}))



import os
import shutil
from pathlib import Path
from importlib.resources import files
import toml
import click
from rich_click import RichCommand
import near_py_tool.click_utils as click_utils

@click.command(cls=RichCommand)
@click.argument('project_dir', default='')
@click.pass_context
def new(ctx, project_dir):
    """Initializes a new project to create a contract"""
    project_dir = click.prompt("Project directory") if project_dir == '' else project_dir
    
    new_project_template = files("near_py_tool") / "assets" / "new-project-template"
    project_abs_path = Path(project_dir).resolve()

    if os.path.isdir(project_abs_path):
        click.echo("Error:")
        click.echo(click.style(f"   Destination `{project_abs_path}` already exists. Refusing to overwrite existing project.", fg="bright_red"))
        return
    
    project_abs_path.mkdir(parents=True, exist_ok=True)
    for src_item in new_project_template.iterdir():
        dst_item = project_abs_path / src_item.name
        if src_item.is_dir():
            shutil.copytree(src_item, dst_item)
        else:
            shutil.copy2(src_item, dst_item)

    pyproject = toml.load(new_project_template / "pyproject.toml")
    pyproject["project"]["name"] = project_abs_path.name
    with open(project_abs_path / "pyproject.toml", "w") as f:
        toml.dump(pyproject, f)

    click.echo(f"""
New project is created at '{project_abs_path}'.

Now you can build, test, and deploy your project using near-py-tool:
 * `near-py-tool build`
 * `near-py-tool deploy`
""")
#Your new project has preconfigured automations for CI and CD, just configure `NEAR_CONTRACT_STAGING_*` and `NEAR_CONTRACT_PRODUCTION_*` variables and secrets on GitHub to enable automatic deployment to staging and production. See more details in `.github/workflow/*` files.
#""")



import click
from rich_click import RichGroup
import near_py_tool.click_utils as click_utils
from near_py_tool.commands.new import new
from near_py_tool.commands.build import build
from near_py_tool.commands.abi import abi
from near_py_tool.commands.create_dev_account import create_dev_account
from near_py_tool.commands.deploy import deploy

@click.group(cls=RichGroup, invoke_without_command=True)
@click.pass_context
def cli(ctx):
    """Python NEAR contract build/deploy tool"""
    click_utils.subcommand_choice(ctx)

cli.add_command(new)
cli.add_command(build)
cli.add_command(abi)
cli.add_command(create_dev_account)
cli.add_command(deploy)

if __name__ == '__main__':
    cli(None)



import click
from rich_click import RichGroup, RichCommand
import near_py_tool.click_utils as click_utils
from pathlib import Path
import json
from near_py_tool.run_command import run_command, is_command_available
from near_py_tool.commands.build import do_build

# todo: decide what to do with extra cmdline arguments inherited form cargo-near which aren't available in near-cli-rs which we are calling to here

def do_deploy(ctx, extra_args):
    if not is_command_available('near'):
        click.echo(click.style("Error: NEAR CLI is required to be installed to deploy a contract", fg='bright_red'))
        click.echo("""
You can install NEAR CLI by running one of the following commands:

  curl --proto '=https' --tlsv1.2 -LsSf https://github.com/near/near-cli-rs/releases/download/v0.18.0/near-cli-rs-installer.sh | sh
  
or 
  
  powershell -ExecutionPolicy ByPass -c "irm https://github.com/near/near-cli-rs/releases/download/v0.18.0/near-cli-rs-installer.ps1 | iex"
  
or 
  
  npm install near-cli-rs@0.18.0
  
or downloading the binaries from https://github.com/near/near-cli-rs/releases/
""")
    params = click_utils.all_parent_command_params(ctx)
    project_dir = params.get('deploy', {}).get('project_dir')
    rebuid_all = params.get('build-non-reproducible-wasm', {}).get('rebuild_all', False)
    project_path = Path(project_dir).resolve()
    project_name = project_path.name
    
    wasm_path = project_path / "build" / Path(project_name).with_suffix(".wasm")    
    do_build(project_path, rebuid_all)

    account_id = (params.get('build-non-reproducible-wasm') or params.get('build-reproducible-wasm')).get('contract_account_id')
    if account_id == '*':
      account_id = local_keychain_account_ids()[0]
    
    cmdline = ['near', 'contract', 'deploy', account_id, 'use-file', wasm_path]
    cmdline.extend([str(arg) for arg in extra_args])
    
    run_command(cmdline, cwd=project_path)
    
def local_keychain_account_ids():
    try:
        accounts_path = Path("~/.near-credentials/accounts.json").expanduser().resolve()
        with open(accounts_path, "r") as f:
            d = json.load(f)
            return [v['account_id'] for v in d]
    except:
        pass
  
def account_id_prompt(prompt):
    local_accounts = local_keychain_account_ids()
    return click_utils.choice(prompt, local_accounts) if local_accounts else click.prompt(prompt)

@click.group(cls=RichGroup, invoke_without_command=True)
@click.option('--project-dir', default='.')
@click.pass_context
def deploy(ctx, project_dir):
    """Add a new contract code"""
    click_utils.subcommand_choice(ctx)
    
@deploy.group(cls=RichGroup, invoke_without_command=True, context_settings={"ignore_unknown_options": True})
@click.pass_context
@click.argument('contract_account_id')
@click.option('--rebuild-all', is_flag=True, help="Rebuild everything from scratch")
@click.argument("extra_args", nargs=-1)
def build_non_reproducible_wasm(ctx, contract_account_id, rebuild_all, extra_args):
    """Fast and simple build (recommended for use during local development)"""
    ctx.params['contract_account_id'] = account_id_prompt("Contract account ID") if not contract_account_id else contract_account_id
    do_deploy(ctx, extra_args)

@deploy.group(cls=RichGroup, invoke_without_command=True, context_settings={"ignore_unknown_options": True})
@click.pass_context
@click.argument('contract_account_id')
@click.argument("extra_args", nargs=-1)
def build_reproducible_wasm(ctx, contract_account_id, extra_args):
    """Build requires [reproducible_build] section in Cargo.toml, and all changes committed and pushed to git (recommended for the production release)"""
    ctx.params['contract_account_id'] = account_id_prompt("Contract account ID") if not contract_account_id else contract_account_id
    do_deploy(ctx, extra_args)



import ast
import os
import sys
import shutil
from pathlib import Path
from importlib.resources import files
import toml
import click
from rich_click import RichGroup, RichCommand
import near_py_tool.click_utils as click_utils
from near_py_tool.run_command import run_command, is_command_available

def get_near_exports_from_file(file_path):
    with open(file_path, "r") as file:
        tree = ast.parse(file.read(), filename=file_path)

    near_exports = set()
    for node in ast.walk(tree):
        if isinstance(node, ast.FunctionDef):
            for d in node.decorator_list:
                if isinstance(d, ast.Attribute) and isinstance(d.value, ast.Name) and d.attr == 'export' and d.value.id == 'near':
                    near_exports.add(node.name)
        if isinstance(node, ast.FunctionDef) and any(isinstance(d, ast.Name) and d.id == 'near.export' for d in node.decorator_list):
            near_exports.add(node.name)

    return near_exports

def get_imports_from_file(file_path):
    """Extract imported modules from a Python file."""
    with open(file_path, "r") as file:
        tree = ast.parse(file.read(), filename=file_path)

    imports = set()
    for node in ast.walk(tree):
        if isinstance(node, ast.Import):
            for alias in node.names:
                imports.add(alias.name)
        elif isinstance(node, ast.ImportFrom):
            module = node.module
            if module:
                imports.add(module)

    return imports

def has_compiled_extensions(package_name, package_path):
    native_library_paths = []
    for ext in ["so", "pyd", "dll", "dylib"]:
        native_library_paths.append(package_path.rglob(f"*.{ext}"))
    return len(native_library_paths) == 0   

def is_mpy_module(name):
    mpy_modules = [
        'array', 'builtins', 'json', 'os', 'random', 'struct', 'sys'
    ]
    return name in mpy_modules

def is_mpy_lib_package(name):
    mpy_lib_packages = [
        'aiohttp', 'cbor2', 'iperf3', 'pyjwt', 'requests'
    ]
    return name in mpy_lib_packages

mpy_stdlib_packages = [
    'binascii', 'contextlib', 'fnmatch', 'hashlib-sha224', 'hmac', 'keyword', 'os-path', 'pprint', 
    'stat', 'tempfile', 'types', 'warnings', '__future__', 'bisect', 'copy', 'functools', 'hashlib-sha256', 
    'html', 'locale', 'pathlib', 'quopri', 'string', 'textwrap', 'unittest', 'zlib', 'abc', 'cmd', 'curses.ascii', 
    'gzip', 'hashlib-sha384', 'inspect', 'logging', 'pickle', 'random', 'struct', 'threading', 'unittest-discover',
    'argparse', 'collections', 'datetime', 'hashlib', 'hashlib-sha512', 'io', 'operator', 'pkg_resources', 'shutil', 
    'tarfile', 'time', 'uu', 'base64', 'collections-defaultdict', 'errno', 'hashlib-core', 'heapq', 'itertools', 'os', 
    'pkgutil', 'ssl', 'tarfile-write', 'traceback', 'venv'
]

near_module_name = 'near'

def is_external_package(name):
    return not is_mpy_module(name) and not is_mpy_lib_package(name) and name not in mpy_stdlib_packages and name != near_module_name

def generate_manifest(contract_path, package_paths, manifest_path):
    with open(manifest_path, "w") as o:
        o.write("# THIS FILE IS GENERATED, DO NOT EDIT\n\n")
        for module in mpy_stdlib_packages:
            o.write(f'require("{module}")\n')
        for mod in ["typing", "typing_extensions"]:
            o.write(f'module("{mod}.py", base_path="$(PORT_DIR)/extra/typing")\n')
        for module, path in package_paths.items():
            if is_mpy_lib_package(module):
                o.write(f'require("{module}")\n')
            elif not is_mpy_module(module):
                o.write(f'package("{module}", base_path="{path.parent}")\n')
        o.write(f'module("{contract_path.name}", base_path="{contract_path.parent}")\n')

def generate_export_wrappers(contract_path, exports, export_wrappers_path):
    with open(export_wrappers_path, "w") as o:
        o.write("/* THIS FILE IS GENERATED, DO NOT EDIT */\n\n")
        o.write("void run_frozen_fn(const char *file_name, const char *fn_name);\n\n")
        for export in exports:
            o.write(f'void {export}() \u007b\n  run_frozen_fn("{contract_path.name}", "{export}");\n\u007d\n\n');

def get_venv_package_paths(file_path, venv_path):
    """Get paths of all packages imported by a Python file."""
    imports = get_imports_from_file(file_path)
    package_paths = {}
    glob_pattern_base = "lib/site-packages" if (venv_path / 'lib' / 'site-packages').is_dir() else "lib/python*.*/site-packages"
    for module_name in imports:
        if is_external_package(module_name):
          paths = list(venv_path.glob(f"{glob_pattern_base}/{module_name}"))
          if len(paths) == 0:
              click.echo(f"Warning: module {module_name} path wasn't resolved; is it installed in the current venv at {venv_path}?")
          elif len(paths) > 1:
              click.echo(f"Warning: module {module_name} has multiple candidate paths: {paths}")
          for path in paths:
              package_paths[module_name] = path

    return package_paths

def install_pyproject_dependencies(project_path, venv_path):
    with open(project_path / "pyproject.toml", "r") as file:
        pyproject_data = toml.load(file)
    for package in pyproject_data.get("project", {}).get("dependencies", {}):
        click.echo(f"Installing {package} into {venv_path}...")
        pip_path = venv_path / "bin" / "pip"
        if not pip_path.is_file():
            pip_path = venv_path / "Scripts" / "pip.exe"
            if not pip_path.is_file():
                click.echo(click.style(f"Error: build Python venv doesn't have pip installed", fg='bright_red'))
                sys.exit(1)
        run_command([pip_path, "install", package, "--disable-pip-version-check"], cwd=project_path)

def do_build(project_dir, rebuild_all):
    project_path = Path(project_dir).resolve()
    print(f"project_path: {project_path}")
    project_name = project_path.name
    build_path = project_path / "build"
    mpy_cross_build_path = project_path / "build" / "mpy-cross"
    venv_path = build_path / ".venv"
    mpy_cross_path = files("near_py_tool") / "assets" / "micropython" / "mpy-cross"
    mpy_port_path = files("near_py_tool") / "assets" / "micropython" / "ports" / "webassembly-near"
    contract_path = project_path / "contract.py"
    if not contract_path.is_file():
        click.echo(click.style(f"Error: contract file {contract_path} doesn't exist", fg='bright_red'))
        sys.exit(1)
        
    if not is_command_available('emcc'):
        click.echo(click.style("Error: Emscripten C to WASM compiler is required for building Python NEAR contracts", fg='bright_red'))
        click.echo("""
You can install Emscripten via a package manager or by doing the following:

  git clone https://github.com/emscripten-core/emsdk.git
  cd emsdk
  ./emsdk install latest
  ./emsdk activate latest
  source ./emsdk_env.sh
                               
""")
        sys.exit(1)

    if not is_command_available('make'):
        click.echo(click.style("Error: make is required for building Python NEAR contracts", fg='bright_red'))
        click.echo("Please install make via a package manager before continuing")
        sys.exit(1)
    
    if rebuild_all:
        click.echo(f"Removing build directory {build_path} to perform a clean build")
        try:
            shutil.rmtree(build_path)
        except Exception:
            pass

    build_path.mkdir(parents=True, exist_ok=True)

    # todo: check for uv and emcc presence and offer installation if missing 
    # https://emscripten.org/docs/getting_started/downloads.html

    # click.echo(f"Running `uv sync` in {project_path}...")
    # run_command(['uv', "sync"], cwd=project_path)

    if not venv_path.is_dir():
        click.echo(f"Creating a venv in {venv_path}...")
        run_command(['python' if is_command_available('python') else 'python3', "-m", "venv", venv_path], cwd=project_path)
      
    install_pyproject_dependencies(project_path, venv_path)

    package_paths = get_venv_package_paths(contract_path, venv_path)

    for module, path in package_paths.items():
        if has_compiled_extensions(module, path):
            click.echo(click.style(f"Warning: required module {module} at {path} has compiled extensions and might not work correctly in WASM environment", fg="bright_red"))

    exports = list(get_near_exports_from_file(contract_path))
    click.echo(f"The contract WASM will export the following methods: {exports}")

    generate_manifest(contract_path, package_paths, build_path / "manifest.py")
    generate_export_wrappers(contract_path, exports, build_path / "export_wrappers.c")
    try:
        os.unlink(build_path / 'frozen_content.c')  # force frozen content rebuilt every time
    except Exception:
        pass

    contract_wasm_path = build_path / Path(project_name).with_suffix(".wasm")
    
    run_command(['make', "-C", mpy_cross_path,
                 f"BUILD={mpy_cross_build_path}"],
                cwd=project_path)

    run_command(['make', "-C", mpy_port_path,
                 f"BUILD={build_path}",
                 f"MICROPY_MPYCROSS={mpy_cross_build_path}/mpy-cross",
                 f"MICROPY_MPYCROSS_DEPENDENCY={mpy_cross_build_path}/mpy-cross",
                 f"FROZEN_MANIFEST={build_path / 'manifest.py'}",
                 f"SRC_C_GENERATED={build_path / 'export_wrappers.c'}",
                 f"EXPORTED_FUNCTIONS={','.join(['_' + e for e in exports])}",
                 f"OUTPUT_WASM={contract_wasm_path}"],
                cwd=project_path)
             
    click.echo(f"Contract WASM file was build successfully and is located at {contract_wasm_path}")

@click.group(cls=RichGroup, invoke_without_command=True)
@click.pass_context
def build(ctx):
    """Build a NEAR contract with embedded ABI"""
    click_utils.subcommand_choice(ctx)

@build.command(cls=RichCommand)
@click.option('--project-dir', default='.')
@click.option('--rebuild-all', is_flag=True, help="Rebuild everything from scratch")
@click.pass_context
def non_reproducible_wasm(ctx, project_dir, rebuild_all):
    """Fast and simple (recommended for use during local development)"""
    do_build(project_dir, rebuild_all)

@build.command(cls=RichCommand)
@click.option('--project-dir', default='.')
@click.pass_context
def reproducible_wasm(ctx, project_dir):
    """Requires `[reproducible_build]` section in pyproject.toml, and all changes committed to git (recommended for the production release)"""
    do_build(project_dir, True)



import click
import questionary

def subcommand_choice(ctx):
    if ctx.invoked_subcommand is None:
        command = ctx.command
        subcommands = [cmd for cmd in command.commands.values() if not cmd.hidden]
        if not subcommands:
            click.echo(command.help)
            ctx.exit()
        subcommand = questionary.select(
            "Choose a command:",
            choices=[
                questionary.Choice(
                    f"{cmd.name} - {cmd.get_short_help_str()}",
                    value=cmd
                ) for cmd in subcommands
            ],
            style=questionary.Style([
                ('qmark', 'fg:cyan bold'),
                ('selected', 'fg:cyan bold'),
                ('pointer', 'fg:yellow bold'),
            ]),
            use_arrow_keys=True
        ).ask() if len(subcommands) > 1 else subcommands[0]
        if subcommand:
            ctx.invoke(subcommand)

def choice(prompt, values):
    return questionary.select(
        prompt,
        choices=[questionary.Choice(v, value=v) for v in values],
        style=questionary.Style([
            ('qmark', 'fg:cyan bold'),
            ('selected', 'fg:cyan bold'),
            ('pointer', 'fg:yellow bold'),
        ]),
        use_arrow_keys=True).ask()
  

def all_parent_command_params(ctx):
    cmd_params = {}
    while ctx:
        cmd_params[ctx.command.name] = ctx.params
        ctx = ctx.parent
    return cmd_params



import randomname
import click
from rich_click import RichGroup, RichCommand
import near_py_tool.click_utils as click_utils
from near_py_tool.run_command import run_command, is_command_available

def is_account_id_available(account_id, network):
    return run_command(['near', 'account', 'view-account-summary', account_id, 'network-config', network, 'now'], fail_on_nonzero_exit_code=False) != 0

def create_account(ctx, extra_args):
    if not is_command_available('near'):
        click.echo(click.style("Error: NEAR CLI is required to be installed to create a new dev account", fg='bright_red'))
        click.echo("""
You can install NEAR CLI by running one of the following commands:

  curl --proto '=https' --tlsv1.2 -LsSf https://github.com/near/near-cli-rs/releases/download/v0.18.0/near-cli-rs-installer.sh | sh
  
or 
  
  powershell -ExecutionPolicy ByPass -c "irm https://github.com/near/near-cli-rs/releases/download/v0.18.0/near-cli-rs-installer.ps1 | iex"
  
or 
  
  npm install near-cli-rs@0.18.0
  
or downloading the binaries from https://github.com/near/near-cli-rs/releases/
""")
    params = click_utils.all_parent_command_params(ctx)
    account_id = (params.get('use-random-account-id') or params.get('use-specific-account-id')).get('new_account_id')
    cmdline = ['near', 'account', 'create-account', 'sponsor-by-faucet-service', account_id]
    cmdline.extend([str(arg) for arg in extra_args])
    run_command(cmdline)
    
@click.group(cls=RichGroup, invoke_without_command=True)
@click.pass_context
def create_dev_account(ctx):
    """Create a development account using the faucet service sponsor to receive some NEAR tokens (testnet only)"""
    click_utils.subcommand_choice(ctx)

@create_dev_account.group(cls=RichGroup, invoke_without_command=True, context_settings={"ignore_unknown_options": True})
@click.pass_context
@click.argument("extra_args", nargs=-1)
def use_random_account_id(ctx, extra_args):
    """I would like to create a random account (useful for quick start development)"""
    new_account_id = f"{randomname.get_name()}.testnet"
    # while not is_account_id_available(new_account_id, 'testnet'):
    #   new_account_id = f"{randomname.get_name()}.testnet"
    click.echo(f"Generated account name: {new_account_id}")
    ctx.params['new_account_id'] = new_account_id
    create_account(ctx, extra_args)
    
@create_dev_account.group(cls=RichGroup, invoke_without_command=True, context_settings={"ignore_unknown_options": True})
@click.pass_context
@click.argument('new_account_id')
@click.argument("extra_args", nargs=-1)
def use_specific_account_id(ctx, new_account_id, extra_args):
    """I would like to create a specific account"""
    ctx.params['new_account_id'] = click.prompt("Account ID") if not new_account_id else new_account_id
    create_account(ctx, extra_args)



import click
from rich_click import RichCommand

@click.command(cls=RichCommand)
@click.option('--locked', is_flag=True)
@click.option('--no-doc', is_flag=True)
@click.option('--compact-abi', is_flag=True)
@click.option('--out-dir', metavar='<OUT_DIR>')
@click.option('--manifest-path', metavar='<MANIFEST_PATH>')
def abi(locked, no_doc, compact_abi, out_dir, manifest_path):
    """Generates ABI for the contract"""
    pass


import sys
import subprocess
import shutil
import click

def is_command_available(command_name):
    return shutil.which(command_name) is not None

def run_command(cmd, fail_on_nonzero_exit_code=True, cwd=None):
    str_cmd = [str(c) for c in cmd]
    exit_code = subprocess.run(str_cmd, cwd=cwd).returncode
    if exit_code != 0 and fail_on_nonzero_exit_code:
        click.echo(click.style(f"Error: command `{' '.join(str_cmd)}` returned {exit_code}", fg='bright_red'))
        sys.exit(1)
    return exit_code
